@@ -2,7 +2,7 @@
 
   compar.c -
 
-  $Author$
+  $Author: naruse $
   created at: Thu Aug 26 14:39:48 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
