@@ -11,6 +11,18 @@
 
 VALUE cFiddleFunction;
 
+#define MAX_ARGS (SIZE_MAX / (sizeof(void *) + sizeof(fiddle_generic)) - 1)
+
+#define Check_Max_Args(name, len) \
+    if ((size_t)(len) < MAX_ARGS) { \
+	/* OK */ \
+    } \
+    else { \
+	rb_raise(rb_eTypeError, \
+		 name" is so large that it can cause integer overflow (%d)", \
+		 (len)); \
+    }
+
 static void
 deallocate(void *p)
 {
@@ -84,6 +96,7 @@ initialize(int argc, VALUE argv[], VALUE self)
     if(NIL_P(abi)) abi = INT2NUM(FFI_DEFAULT_ABI);
 
     Check_Type(args, T_ARRAY);
+    Check_Max_Args("args", RARRAY_LENINT(args));
 
     rb_iv_set(self, "@ptr", ptr);
     rb_iv_set(self, "@args", args);
@@ -124,11 +137,13 @@ function_call(int argc, VALUE argv[], VALUE self)
     void **values;
     VALUE cfunc, types, cPointer;
     int i;
+    VALUE alloc_buffer = 0;
 
     cfunc    = rb_iv_get(self, "@ptr");
     types    = rb_iv_get(self, "@args");
     cPointer = rb_const_get(mFiddle, rb_intern("Pointer"));
 
+    Check_Max_Args("number of arguments", argc);
     if(argc != RARRAY_LENINT(types)) {
 	rb_raise(rb_eArgError, "wrong number of arguments (%d for %d)",
 		argc, RARRAY_LENINT(types));
@@ -145,8 +160,9 @@ function_call(int argc, VALUE argv[], VALUE self)
 	}
     }
 
-    values = xcalloc((size_t)argc + 1, (size_t)sizeof(void *));
-    generic_args = xcalloc((size_t)argc, (size_t)sizeof(fiddle_generic));
+    generic_args = ALLOCV(alloc_buffer,
+	(size_t)(argc + 1) * sizeof(void *) + (size_t)argc * sizeof(fiddle_generic));
+    values = (void **)((char *)generic_args + (size_t)argc * sizeof(fiddle_generic));
 
     for (i = 0; i < argc; i++) {
 	VALUE type = RARRAY_PTR(types)[i];
@@ -154,9 +170,9 @@ function_call(int argc, VALUE argv[], VALUE self)
 
 	if(NUM2INT(type) == TYPE_VOIDP) {
 	    if(NIL_P(src)) {
-		src = INT2NUM(0);
+		src = INT2FIX(0);
 	    } else if(cPointer != CLASS_OF(src)) {
-	        src = rb_funcall(cPointer, rb_intern("[]"), 1, src);
+		src = rb_funcall(cPointer, rb_intern("[]"), 1, src);
 	    }
 	    src = rb_Integer(src);
 	}
@@ -173,8 +189,7 @@ function_call(int argc, VALUE argv[], VALUE self)
     rb_funcall(mFiddle, rb_intern("win32_last_error="), 1, INT2NUM(errno));
 #endif
 
-    xfree(values);
-    xfree(generic_args);
+    ALLOCV_END(alloc_buffer);
 
     return GENERIC2VALUE(rb_iv_get(self, "@return_type"), retval);
 }
@@ -209,7 +224,7 @@ Init_fiddle_function(void)
      *
      * === ABI check
      *
-     *   @libc = DL.dlopen "/lib/libc.so.6"
+     *   @libc = Fiddle.dlopen "/lib/libc.so.6"
      *	    #=> #<Fiddle::Handle:0x00000001d7a8d8>
      *   f = Fiddle::Function.new(@libc['strcpy'], [TYPE_VOIDP, TYPE_VOIDP], TYPE_VOIDP)
      *	    #=> #<Fiddle::Function:0x00000001d8ee00>
