@@ -2,7 +2,7 @@
 
   io/wait.c -
 
-  $Author$
+  $Author: nagachika $
   created at: Tue Aug 28 09:08:06 JST 2001
 
   All the files in this distribution are covered under the Ruby's
