@@ -1,4 +1,4 @@
-# -*- coding: euc-jp -*-
+# -*- coding: utf-8 -*-
 #
 # combo.rb --
 #
@@ -21,16 +21,16 @@ base_frame = TkFrame.new($combo_demo).pack(:fill=>:both, :expand=>true)
 
 Ttk::Label.new(base_frame, :font=>$font, :wraplength=>'5i', :justify=>:left,
                :text=><<EOL).pack(:side=>:top, :fill=>:x)
-ʲǤ3ΥܥܥåɽƤޤ\
-ǽΤΤϡȥꥦåȤƱͤˡ\
-ݥȤꡤåꡤפꤹ뤳ȤǤޤ\
-ޤReturnϤиߤͤꥹȤɲä졤\
-ɥåץꥹȤ򤹤뤳ȤǤ褦ˤʤޤ\
-()򲡤ɽ줿ꥹȤ\
-¾θReturn򲡤СͤǤޤ\
-2ܤΥܥܥåͤ˸ꤵƤꡤѹǤޤ\
-3ܤΤΤϥȥꥢԻԤΥɥåץꥹȤ\
-򤹤뤳ȤǽȤʤäƤޤ
+以下では3種類のコンボボックスが表示されています．\
+最初のものは，エントリウィジェットと同じ様に，\
+ポイントしたり，クリックしたり，タイプしたりすることができます．\
+また，Returnキーを入力すれば現在の値がリストに追加され，\
+ドロップダウンリストから選択することができるようになります．\
+↓(下向き矢印)キーを押して表示されたリストから\
+矢印キーで他の候補を選んでReturnキーを押せば，値を選択できます．\
+2番目のコンボボックスは特定の値に固定されており，一切変更できません．\
+3番目のものはオーストラリアの都市のドロップダウンリストから\
+選択することだけが可能となっています．
 EOL
 
 ## variables
@@ -43,7 +43,7 @@ Ttk::Frame.new(base_frame) {|frame|
   sep = Ttk::Separator.new(frame)
   Tk.grid(sep, :columnspan=>4, :row=>0, :sticky=>'ew', :pady=>2)
   TkGrid('x',
-         Ttk::Button.new(frame, :text=>'ѿ',
+         Ttk::Button.new(frame, :text=>'変数参照',
                          :image=>$image['view'], :compound=>:left,
                          :command=>proc{
                            showVars(base_frame,
@@ -51,10 +51,10 @@ Ttk::Frame.new(base_frame) {|frame|
                                     ['secondVariable', secondValue],
                                     ['ozCity', ozCity])
                          }),
-         Ttk::Button.new(frame, :text=>'ɻ',
+         Ttk::Button.new(frame, :text=>'コード参照',
                          :image=>$image['view'], :compound=>:left,
                          :command=>proc{showCode 'combo'}),
-         Ttk::Button.new(frame, :text=>'Ĥ',
+         Ttk::Button.new(frame, :text=>'閉じる',
                          :image=>$image['delete'], :compound=>:left,
                          :command=>proc{
                            $combo_demo.destroy
@@ -68,13 +68,13 @@ Ttk::Frame.new(base_frame) {|frame|
 frame = Ttk::Frame.new(base_frame).pack(:fill=>:both, :expand=>true)
 
 australianCities = [
-  '٥', 'ɥˡ', 'ܥ', 'ѡ', 'ǥ졼',
-  '֥ꥹ١', 'ۥС', '', 'ꥹ ץ󥰥'
+  'キャンベラ', 'シドニー', 'メルボルン', 'パース', 'アデレード',
+  'ブリスベーン', 'ホバート', 'ダーウィン', 'アリス スプリングス'
 ]
 
 
-secondValue.value = 'ѹԲ'
-ozCity.value = 'ɥˡ'
+secondValue.value = '変更不可'
+ozCity.value = 'シドニー'
 
 Tk.pack(Ttk::Labelframe.new(frame, :text=>'Fully Editable'){|f|
           Ttk::Combobox.new(f, :textvariable=>firstValue){|b|
