@@ -2,7 +2,7 @@
 
   tkutil.c -
 
-  $Author$
+  $Author: usa $
   created at: Fri Nov  3 00:47:54 JST 1995
 
 ************************************************/
@@ -1774,10 +1774,10 @@ Init_tkutil(void)
     ID_call = rb_intern("call");
 
     /* --------------------- */
-    cCB_SUBST = rb_define_class_under(mTK, "CallbackSubst", rb_cData);
+    cCB_SUBST = rb_define_class_under(mTK, "CallbackSubst", rb_cObject);
     rb_define_singleton_method(cCB_SUBST, "inspect", cbsubst_inspect, 0);
 
-    cSUBST_INFO = rb_define_class_under(cCB_SUBST, "Info", rb_cData);
+    cSUBST_INFO = rb_define_class_under(cCB_SUBST, "Info", rb_cObject);
     rb_define_singleton_method(cSUBST_INFO, "inspect", substinfo_inspect, 0);
 
     ID_SUBST_INFO = rb_intern("SUBST_INFO");
