@@ -17,17 +17,20 @@ worksheet.Range("D2").value = 20;
 worksheet.Range("A1:B2").value = [["North","South"],
                                   [5.2, 10]];
 
-vals = WIN32OLE_VARIANT.new([["East","West"], 
-                             [8, 20]], 
-                            WIN32OLE::VARIANT::VT_ARRAY) 
+vals = WIN32OLE_VARIANT.new([["East","West"],
+                             [8, 20]],
+                            WIN32OLE::VARIANT::VT_ARRAY)
 worksheet.Range("C1:D2").value = vals
-                                 
+
 range = worksheet.Range("A1:D2");
 range.Select
 chart = workbook.Charts.Add;
 
 workbook.saved = TRUE;
-sleep 0.5
+
+print "Now quit Excel... Please enter."
+gets
+
 application.ActiveWorkbook.Close(0);
 application.Quit();
 
