@@ -1,7 +1,7 @@
 #
 #   irb.rb - irb main module
 #       $Release Version: 0.9.6 $
-#       $Revision$
+#       $Revision: 54322 $
 #       by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
@@ -13,7 +13,6 @@ require "e2mmap"
 require "irb/init"
 require "irb/context"
 require "irb/extend-command"
-#require "irb/workspace"
 
 require "irb/ruby-lex"
 require "irb/input-method"
@@ -138,8 +137,8 @@ STDOUT.sync = true
 # This example can be used in your +.irbrc+
 #
 #     IRB.conf[:PROMPT][:MY_PROMPT] = { # name of prompt mode
-#       :AUTO_INDENT => true            # enables auto-indent mode
-#       :PROMPT_I => nil,		# normal prompt
+#       :AUTO_INDENT => true,           # enables auto-indent mode
+#       :PROMPT_I =>  ">> ",		# simple prompt
 #       :PROMPT_S => nil,		# prompt for continuated strings
 #       :PROMPT_C => nil,		# prompt for continuated statement
 #       :RETURN => "    ==>%s\n"	# format to return value
@@ -158,7 +157,7 @@ STDOUT.sync = true
 #     %m    # to_s of main object (self)
 #     %M    # inspect of main object (self)
 #     %l    # type of string(", ', /, ]), `]' is inner %w[...]
-#     %NNi  # indent level. NN is degits and means as same as printf("%NNd").
+#     %NNi  # indent level. NN is digits and means as same as printf("%NNd").
 #           # It can be ommited
 #     %NNn  # line number.
 #     %%    # %
@@ -337,7 +336,6 @@ STDOUT.sync = true
 #   # quit irb
 #   irb(main):010:0> exit
 module IRB
-  @RCS_ID='-$Id$-'
 
   # An exception raised by IRB.irb_abort
   class Abort < Exception;end
@@ -399,7 +397,6 @@ module IRB
     ensure
       irb_at_exit
     end
-#    print "\n"
   end
 
   # Calls each event hook of IRB.conf[:AT_EXIT] when the current session quits.
@@ -499,7 +496,7 @@ module IRB
           end
           if exc
             print exc.class, ": ", exc, "\n"
-            if exc.backtrace[0] =~ /irb(2)?(\/.*|-.*|\.rb)?:/ && exc.class.to_s !~ /^IRB/ &&
+            if exc.backtrace && exc.backtrace[0] =~ /irb(2)?(\/.*|-.*|\.rb)?:/ && exc.class.to_s !~ /^IRB/ &&
                 !(SyntaxError === exc)
               irb_bug = true
             else
@@ -509,16 +506,18 @@ module IRB
             messages = []
             lasts = []
             levels = 0
-            for m in exc.backtrace
-              m = @context.workspace.filter_backtrace(m) unless irb_bug
-              if m
-                if messages.size < @context.back_trace_limit
-                  messages.push "\tfrom "+m
-                else
-                  lasts.push "\tfrom "+m
-                  if lasts.size > @context.back_trace_limit
-                    lasts.shift
-                    levels += 1
+            if exc.backtrace
+              for m in exc.backtrace
+                m = @context.workspace.filter_backtrace(m) unless irb_bug
+                if m
+                  if messages.size < @context.back_trace_limit
+                    messages.push "\tfrom "+m
+                  else
+                    lasts.push "\tfrom "+m
+                    if lasts.size > @context.back_trace_limit
+                      lasts.shift
+                      levels += 1
+                    end
                   end
                 end
               end
@@ -526,7 +525,7 @@ module IRB
             print messages.join("\n"), "\n"
             unless lasts.empty?
               printf "... %d levels...\n", levels if levels > 0
-              print lasts.join("\n")
+              print lasts.join("\n"), "\n"
             end
             print "Maybe IRB bug!\n" if irb_bug
           end
