@@ -2,7 +2,7 @@
 
   numeric.c -
 
-  $Author$
+  $Author: nagachika $
   created at: Fri Aug 13 18:33:09 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
