@@ -2,7 +2,7 @@
 
   signal.c -
 
-  $Author$
+  $Author: nagachika $
   created at: Tue Dec 20 10:13:44 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -745,7 +745,7 @@ rb_get_next_signal(void)
 
 #if defined SIGSEGV || defined SIGBUS || defined SIGILL || defined SIGFPE
 static const char *received_signal;
-# define clear_received_signal() (void)(received_signal = 0)
+# define clear_received_signal() (void)(ruby_disable_gc = 0, received_signal = 0)
 #else
 # define clear_received_signal() ((void)0)
 #endif
