@@ -2,7 +2,7 @@
 
   sprintf.c -
 
-  $Author: akr $
+  $Author: nagachika $
   created at: Fri Oct 15 10:39:26 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -1055,7 +1055,7 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		}
 		val = rb_obj_as_string(num);
 		len = RSTRING_LEN(val) + zero;
-		if (prec >= len) ++len; /* integer part 0 */
+		if (prec >= len) len = prec + 1; /* integer part 0 */
 		if (sign || (flags&FSPACE)) ++len;
 		if (prec > 0) ++len; /* period */
 		CHECK(len > width ? len : width);
@@ -1095,16 +1095,19 @@ rb_str_format(int argc, const VALUE *argv, VALUE fmt)
 		    done += prec;
 		}
 		if ((flags & FWIDTH) && width > done) {
+		    int fill = ' ';
+		    long shifting = 0;
 		    if (!(flags&FMINUS)) {
-			long i, shifting = (flags&FZERO) ? done - prefix : done;
-			for (i = 1; i <= shifting; i++)
-			    buf[width - i] = buf[done - i];
+			shifting = done;
+			if (flags&FZERO) {
+			    shifting -= prefix;
+			    fill = '0';
+			}
 			blen -= shifting;
-			FILL((flags&FZERO) ? '0' : ' ', width - done);
-			blen += shifting;
-		    } else {
-			FILL(' ', width - done);
+			memmove(&buf[blen + width - done], &buf[blen], shifting);
 		    }
+		    FILL(fill, width - done);
+		    blen += shifting;
 		}
 		RB_GC_GUARD(val);
 		break;
