@@ -318,13 +318,11 @@ class CGITagHelperTest < Test::Unit::TestCase
     assert_match(/^<INPUT .*TYPE="checkbox".*>bb<INPUT .*TYPE="checkbox".*>dd$/,str)
     assert_match(/^<INPUT .*NAME="foo".*>bb<INPUT .*NAME="foo".*>dd$/,str)
     assert_match(/^<INPUT .*>bb<INPUT .*CHECKED.*>dd$/,str)
-    assert_match(/<INPUT .*TYPE="text".*>/,cgi.text_field(:name=>"name",:value=>"value")) if RUBY_VERSION>="1.9"
-    if RUBY_VERSION>="1.9"
-      str=cgi.radio_group("foo",["aa","bb"],["cc","dd",false])
-      assert_match(/^<INPUT .*VALUE="aa".*>bb<INPUT .*VALUE="cc".*>dd$/,str)
-      assert_match(/^<INPUT .*TYPE="radio".*>bb<INPUT .*TYPE="radio".*>dd$/,str)
-      assert_match(/^<INPUT .*NAME="foo".*>bb<INPUT .*NAME="foo".*>dd$/,str)
-    end
+    assert_match(/<INPUT .*TYPE="text".*>/,cgi.text_field(:name=>"name",:value=>"value"))
+    str=cgi.radio_group("foo",["aa","bb"],["cc","dd",false])
+    assert_match(/^<INPUT .*VALUE="aa".*>bb<INPUT .*VALUE="cc".*>dd$/,str)
+    assert_match(/^<INPUT .*TYPE="radio".*>bb<INPUT .*TYPE="radio".*>dd$/,str)
+    assert_match(/^<INPUT .*NAME="foo".*>bb<INPUT .*NAME="foo".*>dd$/,str)
   end
 
 =begin
@@ -349,6 +347,7 @@ class CGITagHelperTest < Test::Unit::TestCase
     assert_equal('<FOOTER></FOOTER>',cgi.footer)
     assert_equal('<ARTICLE></ARTICLE>',cgi.article)
     assert_equal('<SECTION></SECTION>',cgi.section)
+    assert_equal('<!DOCTYPE HTML><HTML BLA="TEST"></HTML>',cgi.html("BLA"=>"TEST"){})
   end
 
 end
