@@ -41,6 +41,14 @@ class TestRDocContext < XrefTestCase
     assert_equal [as], @context.unmatched_alias_lists['#old_name']
   end
 
+  def test_add
+    @context.add RDoc::Extend,  'Ext', 'comment'
+    @context.add RDoc::Include, 'Incl', 'comment'
+
+    refute_empty @context.extends
+    refute_empty @context.includes
+  end
+
   def test_add_alias_method_attr
     top_level = @store.add_file 'file.rb'
 
@@ -613,6 +621,8 @@ class TestRDocContext < XrefTestCase
 
     assert_equal 1,  @c2_c3.<=>(@c2)
     assert_equal(-1, @c2_c3.<=>(@c3))
+
+    assert_nil @c2.<=>(Gem.loaded_specs.values.first)
   end
 
   def test_methods_by_type
@@ -691,6 +701,15 @@ class TestRDocContext < XrefTestCase
     assert_equal [@apub, @aprot, @apriv], @vis.attributes
   end
 
+  def test_remove_invisible_nodoc
+    util_visibilities
+
+    @vis.remove_invisible :nodoc
+
+    assert_equal [@pub, @prot, @priv], @vis.method_list
+    assert_equal [@apub, @aprot, @apriv], @vis.attributes
+  end
+
   def test_remove_invisible_protected
     util_visibilities
 
