@@ -1,6 +1,5 @@
 begin
   require_relative 'dummyparser'
-  require_relative '../ruby/envutil'
   require 'test/unit'
   ripper_test = true
   module TestRipper; end
@@ -9,13 +8,10 @@ end
 
 class TestRipper::ParserEvents < Test::Unit::TestCase
 
-  # should be enabled
   def test_event_coverage
-    dispatched = Ripper::PARSER_EVENTS.map {|event,*| event }
-    dispatched.each do |e|
-      assert_equal true, respond_to?("test_#{e}", true),
-                   "event not tested: #{e.inspect}"
-    end
+    dispatched = Ripper::PARSER_EVENTS
+    tested = self.class.instance_methods(false).grep(/\Atest_(\w+)/) {$1.intern}
+    assert_empty dispatched-tested
   end
 
   def parse(str, nm = nil, &bl)
@@ -24,6 +20,10 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     dp.parse.to_s
   end
 
+  def compile_error(str)
+    parse(str, :compile_error) {|e, msg| return msg}
+  end
+
   def test_program
     thru_program = false
     assert_equal '[void()]', parse('', :on_program) {thru_program = true}
@@ -146,6 +146,12 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     assert_equal true, thru_assoc_new
   end
 
+  def test_assoc_splat
+    thru_assoc_splat = false
+    parse('m(**h)', :on_assoc_splat) {thru_assoc_splat = true}
+    assert_equal true, thru_assoc_splat
+  end
+
   def test_aref_field
     assert_equal '[assign(aref_field(vcall(a),[1]),2)]', parse('a[1]=2')
   end
@@ -173,29 +179,48 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
   end
 
   def test_assign_error
+    # for test_coverage
+  end
+
+  def test_assign_error_backref
     thru_assign_error = false
     parse('$` = 1', :on_assign_error) {thru_assign_error = true}
     assert_equal true, thru_assign_error
     thru_assign_error = false
     parse('$`, _ = 1', :on_assign_error) {thru_assign_error = true}
     assert_equal true, thru_assign_error
+  end
 
+  def test_assign_error_const_qualified
     thru_assign_error = false
     parse('self::X = 1', :on_assign_error) {thru_assign_error = true}
     assert_equal false, thru_assign_error
-    parse('def m\n self::X = 1\nend', :on_assign_error) {thru_assign_error = true}
+    parse("def m\n self::X = 1\nend", :on_assign_error) {thru_assign_error = true}
     assert_equal true, thru_assign_error
+    thru_assign_error = false
+    parse("def m\n self::X, a = 1, 2\nend", :on_assign_error) {thru_assign_error = true}
+    assert_equal true, thru_assign_error
+  end
 
+  def test_assign_error_const
     thru_assign_error = false
     parse('X = 1', :on_assign_error) {thru_assign_error = true}
     assert_equal false, thru_assign_error
-    parse('def m\n X = 1\nend', :on_assign_error) {thru_assign_error = true}
+    parse("def m\n X = 1\nend", :on_assign_error) {thru_assign_error = true}
     assert_equal true, thru_assign_error
+    thru_assign_error = false
+    parse("def m\n X, a = 1, 2\nend", :on_assign_error) {thru_assign_error = true}
+    assert_equal true, thru_assign_error
+  end
 
+  def test_assign_error_const_toplevel
     thru_assign_error = false
     parse('::X = 1', :on_assign_error) {thru_assign_error = true}
     assert_equal false, thru_assign_error
-    parse('def m\n ::X = 1\nend', :on_assign_error) {thru_assign_error = true}
+    parse("def m\n ::X = 1\nend", :on_assign_error) {thru_assign_error = true}
+    assert_equal true, thru_assign_error
+    thru_assign_error = false
+    parse("def m\n ::X, a = 1, 2\nend", :on_assign_error) {thru_assign_error = true}
     assert_equal true, thru_assign_error
   end
 
@@ -220,9 +245,8 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     assert_equal true, thru_begin
   end
 
-  def test_binary
-    thru_binary = nil
-    %w"and or + - * / % ** | ^ & <=> > >= < <= == === != =~ !~ << >> && ||".each do |op|
+  %w"and or + - * / % ** | ^ & <=> > >= < <= == === != =~ !~ << >> && ||".each do |op|
+    define_method("test_binary(#{op})") do
       thru_binary = false
       parse("a #{op} b", :on_binary) {thru_binary = true}
       assert_equal true, thru_binary
@@ -349,14 +373,14 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
   def test_heredoc
     bug1921 = '[ruby-core:24855]'
     thru_heredoc_beg = false
-    tree = parse("<<EOS\nheredoc\nEOS\n", :on_heredoc_beg) {thru_heredoc_beg = true}
+    tree = parse("<""<EOS\nheredoc\nEOS\n", :on_heredoc_beg) {thru_heredoc_beg = true}
     assert_equal true, thru_heredoc_beg
     assert_match(/string_content\(\),heredoc\n/, tree, bug1921)
     heredoc = nil
-    parse("<<EOS\nheredoc1\nheredoc2\nEOS\n", :on_string_add) {|e, n, s| heredoc = s}
+    parse("<""<EOS\nheredoc1\nheredoc2\nEOS\n", :on_string_add) {|e, n, s| heredoc = s}
     assert_equal("heredoc1\nheredoc2\n", heredoc, bug1921)
     heredoc = nil
-    parse("<<-EOS\nheredoc1\nheredoc2\n\tEOS\n", :on_string_add) {|e, n, s| heredoc = s}
+    parse("<""<-EOS\nheredoc1\nheredoc2\n\tEOS\n", :on_string_add) {|e, n, s| heredoc = s}
     assert_equal("heredoc1\nheredoc2\n", heredoc, bug1921)
   end
 
@@ -519,6 +543,10 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     thru_dyna_symbol = false
     parse(':"#{foo}"', :on_dyna_symbol) {thru_dyna_symbol = true}
     assert_equal true, thru_dyna_symbol
+
+    thru_dyna_symbol = false
+    parse('{"#{foo}": 1}', :on_dyna_symbol) {thru_dyna_symbol = true}
+    assert_equal true, thru_dyna_symbol
   end
 
   def test_else
@@ -592,8 +620,8 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
 
   def test_magic_comment
     thru_magic_comment = false
-    parse('# -*- foo:bar -*-', :on_magic_comment) {thru_magic_comment = true}
-    assert_equal true, thru_magic_comment
+    parse('# -*- bug-5753: ruby-dev:44984 -*-', :on_magic_comment) {|*x|thru_magic_comment = x}
+    assert_equal [:on_magic_comment, "bug_5753", "ruby-dev:44984"], thru_magic_comment
   end
 
   def test_method_add_block
@@ -693,12 +721,15 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     thru_opassign = false
     parse('a ||= b', :on_opassign) {thru_opassign = true}
     assert_equal true, thru_opassign
+    thru_opassign = false
+    parse('a::X ||= c 1', :on_opassign) {thru_opassign = true}
+    assert_equal true, thru_opassign
   end
 
   def test_opassign_error
     thru_opassign = []
-    events = [:on_opassign, :on_assign_error]
-    parse('a::X ||= c 1', events) {|a,*b|
+    events = [:on_opassign]
+    parse('$~ ||= 1', events) {|a,*b|
       thru_opassign << a
     }
     assert_equal events, thru_opassign
@@ -720,15 +751,31 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
   end
 
   def test_params
+    arg = nil
     thru_params = false
-    parse('a {||}', :on_params) {thru_params = true}
+    parse('a {||}', :on_params) {|_, *v| thru_params = true; arg = v}
     assert_equal true, thru_params
+    assert_equal [nil, nil, nil, nil, nil, nil, nil], arg
     thru_params = false
-    parse('a {|x|}', :on_params) {thru_params = true}
+    parse('a {|x|}', :on_params) {|_, *v| thru_params = true; arg = v}
     assert_equal true, thru_params
+    assert_equal [["x"], nil, nil, nil, nil, nil, nil], arg
     thru_params = false
-    parse('a {|*x|}', :on_params) {thru_params = true}
+    parse('a {|*x|}', :on_params) {|_, *v| thru_params = true; arg = v}
     assert_equal true, thru_params
+    assert_equal [nil, nil, "*x", nil, nil, nil, nil], arg
+    thru_params = false
+    parse('a {|x: 1|}', :on_params) {|_, *v| thru_params = true; arg = v}
+    assert_equal true, thru_params
+    assert_equal [nil, nil, nil, nil, [["x:", "1"]], nil, nil], arg
+    thru_params = false
+    parse('a {|x:|}', :on_params) {|_, *v| thru_params = true; arg = v}
+    assert_equal true, thru_params
+    assert_equal [nil, nil, nil, nil, [["x:", false]], nil, nil], arg
+    thru_params = false
+    parse('a {|**x|}', :on_params) {|_, *v| thru_params = true; arg = v}
+    assert_equal true, thru_params
+    assert_equal [nil, nil, nil, nil, nil, "x", nil], arg
   end
 
   def test_paren
@@ -749,12 +796,36 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     assert_equal true, thru_qwords_add
   end
 
+  def test_qsymbols_add
+    thru_qsymbols_add = false
+    parse('%i[a]', :on_qsymbols_add) {thru_qsymbols_add = true}
+    assert_equal true, thru_qsymbols_add
+  end
+
+  def test_symbols_add
+    thru_symbols_add = false
+    parse('%I[a]', :on_symbols_add) {thru_symbols_add = true}
+    assert_equal true, thru_symbols_add
+  end
+
   def test_qwords_new
     thru_qwords_new = false
     parse('%w[]', :on_qwords_new) {thru_qwords_new = true}
     assert_equal true, thru_qwords_new
   end
 
+  def test_qsymbols_new
+    thru_qsymbols_new = false
+    parse('%i[]', :on_qsymbols_new) {thru_qsymbols_new = true}
+    assert_equal true, thru_qsymbols_new
+  end
+
+  def test_symbols_new
+    thru_symbols_new = false
+    parse('%I[]', :on_symbols_new) {thru_symbols_new = true}
+    assert_equal true, thru_symbols_new
+  end
+
   def test_redo
     thru_redo = false
     parse('redo', :on_redo) {thru_redo = true}
@@ -783,8 +854,16 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
     thru_rescue = false
     parsed = parse('begin; 1; rescue => e; 2; end', :on_rescue) {thru_rescue = true}
     assert_equal true, thru_rescue
-    assert_match /1.*rescue/, parsed
-    assert_match /rescue\(,var_field\(e\),\[2\]\)/, parsed
+    assert_match(/1.*rescue/, parsed)
+    assert_match(/rescue\(,var_field\(e\),\[2\]\)/, parsed)
+  end
+
+  def test_rescue_class
+    thru_rescue = false
+    parsed = parse('begin; 1; rescue RuntimeError => e; 2; end', :on_rescue) {thru_rescue = true}
+    assert_equal true, thru_rescue
+    assert_match(/1.*rescue/, parsed)
+    assert_match(/rescue\(\[ref\(RuntimeError\)\],var_field\(e\),\[2\]\)/, parsed)
   end
 
   def test_rescue_mod
@@ -1116,7 +1195,6 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
   def test_local_variables
     cmd = 'command(w,[regexp_literal(regexp_add(regexp_new(),25 # ),/)])'
     div = 'binary(ref(w),/,25)'
-    var = '[w]'
     bug1939 = '[ruby-core:24923]'
 
     assert_equal("[#{cmd}]", parse('w /25 # /'), bug1939)
@@ -1135,8 +1213,20 @@ class TestRipper::ParserEvents < Test::Unit::TestCase
   end
 
   def test_unterminated_regexp
-    compile_error = false
-    parse('/', :compile_error) {|e, msg| compile_error = msg}
-    assert_equal("unterminated regexp meets end of file", compile_error)
+    assert_equal("unterminated regexp meets end of file", compile_error('/'))
+  end
+
+  def test_invalid_instance_variable_name
+    assert_equal("`@1' is not allowed as an instance variable name", compile_error('@1'))
+    assert_equal("`@%' is not allowed as an instance variable name", compile_error('@%'))
+  end
+
+  def test_invalid_class_variable_name
+    assert_equal("`@@1' is not allowed as a class variable name", compile_error('@@1'))
+    assert_equal("`@@%' is not allowed as a class variable name", compile_error('@@%'))
+  end
+
+  def test_invalid_global_variable_name
+    assert_equal("`$%' is not allowed as a global variable name", compile_error('$%'))
   end
 end if ripper_test
