@@ -139,6 +139,7 @@ class TestDir < Test::Unit::TestCase
 
     assert_equal((?a..?z).step(2).map {|f| File.join(File.join(@root, f), "") }.sort,
                  Dir.glob(File.join(@root, "*/")).sort)
+    assert_equal([File.join(@root, '//a')], Dir.glob(@root + '//a'))
 
     FileUtils.touch(File.join(@root, "{}"))
     assert_equal(%w({} a).map{|f| File.join(@root, f) },
@@ -228,6 +229,37 @@ class TestDir < Test::Unit::TestCase
     assert_empty(Dir.glob(File.join(@root, "<")), bug8597)
   end
 
+  def test_glob_cases
+    feature5994 = "[ruby-core:42469] [Feature #5994]"
+    feature5994 << "\nDir.glob should return the filename with actual cases on the filesystem"
+    Dir.chdir(File.join(@root, "a")) do
+      open("FileWithCases", "w") {}
+      return unless File.exist?("filewithcases")
+      assert_equal(%w"FileWithCases", Dir.glob("filewithcases"), feature5994)
+    end
+    Dir.chdir(@root) do
+      assert_equal(%w"a/FileWithCases", Dir.glob("A/filewithcases"), feature5994)
+    end
+  end
+
+  def test_glob_super_root
+    bug9648 = '[ruby-core:61552] [Bug #9648]'
+    roots = Dir.glob("/*")
+    assert_equal(roots.map {|n| "/..#{n}"}, Dir.glob("/../*"), bug9648)
+  end
+
+  if /mswin|mingw/ =~ RUBY_PLATFORM
+    def test_glob_legacy_short_name
+      bug10819 = '[ruby-core:67954] [Bug #10819]'
+      skip unless /\A\w:/ =~ ENV["ProgramFiles"]
+      short = "#$&/PROGRA~1"
+      skip unless File.directory?(short)
+      entries = Dir.glob("#{short}/Common*")
+      assert_not_empty(entries, bug10819)
+      assert_equal(Dir.glob("#{File.expand_path(short)}/Common*"), entries, bug10819)
+    end
+  end
+
   def test_home
     env_home = ENV["HOME"]
     env_logdir = ENV["LOGDIR"]
@@ -266,4 +298,31 @@ class TestDir < Test::Unit::TestCase
       end
     end
   end
+
+  def test_fileno
+    Dir.open(".") {|d|
+      if d.respond_to? :fileno
+        assert_kind_of(Integer, d.fileno)
+      else
+        assert_raise(NotImplementedError) { d.fileno }
+      end
+    }
+  end
+
+  def test_insecure_chdir
+    assert_raise(SecurityError) do
+      proc do
+        $SAFE=3
+        Dir.chdir("/")
+      end.call
+    end
+    m = "\u{79fb 52d5}"
+    d = Class.new(Dir) {singleton_class.class_eval {alias_method m, :chdir}}
+    assert_raise_with_message(SecurityError, /#{m}/) do
+      proc do
+        $SAFE=3
+        d.__send__(m, "/")
+      end.call
+    end
+  end
 end
