@@ -1,5 +1,4 @@
 require 'test/unit'
-require_relative 'envutil'
 
 class TestVariable < Test::Unit::TestCase
   class Gods
@@ -29,7 +28,9 @@ class TestVariable < Test::Unit::TestCase
     @@rule = "Cronus"			# modifies @@rule in Gods
     include Olympians
     def ruler4
-      @@rule
+      EnvUtil.suppress_warning {
+        @@rule
+      }
     end
   end
 
@@ -65,6 +66,7 @@ class TestVariable < Test::Unit::TestCase
   def test_local_variables
     lvar = 1
     assert_instance_of(Symbol, local_variables[0], "[ruby-dev:34008]")
+    lvar
   end
 
   def test_local_variables2
@@ -72,6 +74,7 @@ class TestVariable < Test::Unit::TestCase
     proc do |y|
       assert_equal([:x, :y], local_variables.sort)
     end.call
+    x
   end
 
   def test_local_variables3
@@ -81,18 +84,19 @@ class TestVariable < Test::Unit::TestCase
         assert_equal([:x, :y, :z], local_variables.sort)
       end
     end.call
+    x
   end
 
   def test_shadowing_local_variables
     bug9486 = '[ruby-core:60501] [Bug #9486]'
     x = tap {|x| break local_variables}
-    assert_equal([:x, :bug9486, :x], x)
+    assert_equal([:x, :bug9486], x)
   end
 
   def test_shadowing_block_local_variables
     bug9486 = '[ruby-core:60501] [Bug #9486]'
     x = tap {|;x| break local_variables}
-    assert_equal([:x, :bug9486, :x], x)
+    assert_equal([:x, :bug9486], x)
   end
 
   def test_global_variable_0
@@ -100,10 +104,29 @@ class TestVariable < Test::Unit::TestCase
   end
 
   def test_global_variable_poped
-    assert_nothing_raised { eval("$foo; 1") }
+    assert_nothing_raised {
+      EnvUtil.suppress_warning {
+        eval("$foo; 1")
+      }
+    }
   end
 
   def test_constant_poped
-    assert_nothing_raised { eval("TestVariable::Gods; 1") }
+    assert_nothing_raised {
+      EnvUtil.suppress_warning {
+        eval("TestVariable::Gods; 1")
+      }
+    }
+  end
+
+  def test_local_variables_with_kwarg
+    bug11674 = '[ruby-core:71437] [Bug #11674]'
+    v = with_kwargs_11(v1:1,v2:2,v3:3,v4:4,v5:5,v6:6,v7:7,v8:8,v9:9,v10:10,v11:11)
+    assert_equal(%i(v1 v2 v3 v4 v5 v6 v7 v8 v9 v10 v11), v, bug11674)
+  end
+
+  private
+  def with_kwargs_11(v1:, v2:, v3:, v4:, v5:, v6:, v7:, v8:, v9:, v10:, v11:)
+    local_variables
   end
 end
