@@ -1,20 +1,31 @@
 require "rubygems"
 require "rubygems/test_case"
 require "rubygems/commands/help_command"
-require "rubygems/format"
+require "rubygems/package"
 require "rubygems/command_manager"
+require File.expand_path('../rubygems_plugin', __FILE__)
 
 class TestGemCommandsHelpCommand < Gem::TestCase
   def setup
     super
 
     @cmd = Gem::Commands::HelpCommand.new
+
+    load File.expand_path('../rubygems_plugin.rb', __FILE__) unless
+      Gem::Commands.const_defined? :InterruptCommand
   end
 
   def test_gem_help_bad
     util_gem 'bad' do |out, err|
       assert_equal('', out)
-      assert_match(/Unknown command bad. Try gem help commands\n/, err)
+      assert_match "Unknown command bad", err
+    end
+  end
+
+  def test_gem_help_gem_dependencies
+    util_gem 'gem_dependencies' do |out, err|
+      assert_match 'gem.deps.rb', out
+      assert_equal '', err
     end
   end
 
@@ -32,7 +43,12 @@ class TestGemCommandsHelpCommand < Gem::TestCase
       mgr.command_names.each do |cmd|
         assert_match(/\s+#{cmd}\s+\S+/, out)
       end
-      assert_equal '', err
+
+      if defined?(OpenSSL::SSL) then
+        assert_empty err
+
+        refute_match 'No command found for ', out
+      end
     end
   end
 
