@@ -28,14 +28,30 @@ class TestGemImpossibleDependenciesError < Gem::TestCase
 
     expected = <<-EXPECTED
 rye-0.9.8 requires net-ssh (>= 2.0.13) but it conflicted:
-  Activated net-ssh-2.6.5 via:
-    net-ssh-2.6.5 (>= 2.0.13), rye-0.9.8 (= 0.9.8)
-  instead of (~> 2.2.2) via:
-    net-ssh-2.6.5 (>= 2.0.13), rye-0.9.8 (= 0.9.8)
-  Activated net-ssh-2.2.2 via:
-    net-ssh-2.2.2 (>= 2.0.13), rye-0.9.8 (= 0.9.8)
-  instead of (>= 2.6.5) via:
-    net-ssh-2.2.2 (>= 2.0.13), rye-0.9.8 (= 0.9.8)
+  Activated net-ssh-2.6.5
+  which does not match conflicting dependency (~> 2.2.2)
+
+  Conflicting dependency chains:
+    rye (= 0.9.8), 0.9.8 activated, depends on
+    net-ssh (>= 2.0.13), 2.6.5 activated
+
+  versus:
+    rye (= 0.9.8), 0.9.8 activated, depends on
+    net-ssh (>= 2.0.13), 2.6.5 activated, depends on
+    net-ssh (~> 2.2.2)
+
+  Activated net-ssh-2.2.2
+  which does not match conflicting dependency (>= 2.6.5)
+
+  Conflicting dependency chains:
+    rye (= 0.9.8), 0.9.8 activated, depends on
+    net-ssh (>= 2.0.13), 2.2.2 activated
+
+  versus:
+    rye (= 0.9.8), 0.9.8 activated, depends on
+    net-ssh (>= 2.0.13), 2.2.2 activated, depends on
+    net-ssh (>= 2.6.5)
+
     EXPECTED
 
     assert_equal expected, error.message
