@@ -415,7 +415,6 @@ if defined? Zlib
       z = Zlib::Inflate.new
       assert_raise(Zlib::DataError) { z << "\0" * 100 + s }
       assert_equal(true, z.sync(""))
-      #assert_equal(true, z.sync_point?)
 
       z = Zlib::Inflate.new
       assert_equal(false, z.sync("\0" * 100))
@@ -423,7 +422,6 @@ if defined? Zlib
 
       z = Zlib::Inflate.new
       assert_equal(true, z.sync("\0" * 100 + s))
-      #assert_equal(true, z.sync_point?)
     end
 
     def test_set_dictionary
@@ -1046,6 +1044,11 @@ if defined? Zlib
         assert_equal(0x02820145, Zlib.adler32_combine(one, two, 1))
       rescue NotImplementedError
         skip "adler32_combine is not implemented"
+      rescue Minitest::Assertion
+        if /aix/ =~ RUBY_PLATFORM
+          skip "zconf.h in zlib does not handle _LARGE_FILES in AIX. Skip until it is fixed"
+        end
+        raise $!
       end
     end
 
@@ -1063,6 +1066,11 @@ if defined? Zlib
         assert_equal(0x8c736521, Zlib.crc32_combine(one, two, 1))
       rescue NotImplementedError
         skip "crc32_combine is not implemented"
+      rescue Minitest::Assertion
+        if /aix/ =~ RUBY_PLATFORM
+          skip "zconf.h in zlib does not handle _LARGE_FILES in AIX. Skip until it is fixed"
+        end
+        raise $!
       end
     end
 
